<?php

App::uses('Route', 'Routable.Model');

class RoutesShell extends AppShell {

	public function main() {
		echo "Please run \"rebuild [[MODEL]]\"	";
	}

	public function rebuild() {
		if (isset($this->args[0])) {

			if (strpos($this->args[0], ',') !== false) {
				$models = explode(',', $this->args[0]);
			} else {
				$models = [$this->args[0]];
			}

			if (!empty($models)) {
				$Route = EvClassRegistry::init('Routable.Route');

				foreach ($models as $model) {
					$Model = EvClassRegistry::init($model);

					if ($Model) {
						$results = $Model->find('all');
						$this->out(__('Creating new routes for %s', $model));

						foreach ($results as $result) {
							$this->_createRouteForModel($result, $Model, $Route);
						}

						//Create Route task

						$QueuedTask = EvClassRegistry::init('Queue.QueuedTask');

						$jobExists = $QueuedTask->find(
							'first',
							[
								'conditions' => [
									'QueuedTask.jobtype' => 'Route',
									'QueuedTask.fetched' => null
								]
							]
						);

						if (empty($jobExists)) {
							$QueuedTask->createJob(
								'Route'
							);
						}
					} else {
						$this->out(__('Could not find model for %s', $model));
					}
				}
			} else {
				$this->out(__('Could not find models to rebuild'));
			}

			$this->out('');
			$this->out('Route building complete.');
			$this->hr();
		} else {

			$this->out('Please specify a model');
		}
	}

	protected function _createRouteForModel($modelItem, $Model, $Route) {
		if ($Model->hasBehavior('Routable.Routable')) {

			$Model->data = $modelItem;

			$data['Route']['actual'] = $Model->actual($modelItem);
			// $data['Route']['alias'] = $Model->alias(null);

			// create a unique alias
			$baseAlias = $Model->alias(null);

			$suffix = 0;

			do {
				$suffix++;

				$alias = $baseAlias;
				if ($suffix > 1) {
					$alias = $Model->alias($suffix);
				}

				$found = $Route->findByAlias($alias);

			} while ($found);

			$data['Route']['alias'] = $alias;

			$Route = new Route();

			// Create/Update the route.

			// @TODO: There's a potential race condition here, someone else may
			// have just inserted the same route (unlikely). In this scenariom the
			// $Model will have saved, but the route won't.
			$Route->create();
			$Route->save(
				$data,
				[
					'callbacks' => false
				]
			);
		}
	}
}
